/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkCaptionRepresentation.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include "vtkCaptionRepresentation.h"
#include "vtkCaptionActor2D.h"
#include "vtkPointHandleRepresentation3D.h"
#include "vtkConeSource.h"
#include "vtkTextProperty.h"
#include "vtkObjectFactory.h"
#include "vtkRenderer.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkPointWidget.h"
#include "vtkRenderWindow.h"

vtkCxxRevisionMacro(vtkCaptionRepresentation, "$Revision: 1.13 $");
vtkStandardNewMacro(vtkCaptionRepresentation);

vtkCxxSetObjectMacro(vtkCaptionRepresentation,CaptionActor,vtkCaptionActor2D);

//-------------------------------------------------------------------------
vtkCaptionRepresentation::vtkCaptionRepresentation()
{
  this->AnchorRepresentation = vtkPointHandleRepresentation3D::New();
  this->AnchorRepresentation->AllOff();
  this->AnchorRepresentation->SetHotSpotSize(1.0);
  this->AnchorRepresentation->SetPlaceFactor(1.0);
  this->AnchorRepresentation->TranslationModeOn();
  this->AnchorRepresentation->ActiveRepresentationOn();

  this->CaptionActor = vtkCaptionActor2D::New();
  this->CaptionActor->GetPositionCoordinate()->SetCoordinateSystemToDisplay();
  this->CaptionActor->GetPositionCoordinate()->SetReferenceCoordinate(0);
  this->CaptionActor->GetPosition2Coordinate()->SetCoordinateSystemToDisplay();
  this->CaptionActor->GetPosition2Coordinate()->SetReferenceCoordinate(0);
  this->CaptionActor->GetPositionCoordinate()->SetValue(10,10);
  this->CaptionActor->GetPosition2Coordinate()->SetValue(20,20);
  this->CaptionActor->SetCaption("Caption Here");
  this->CaptionActor->SetAttachmentPoint(0,0,0);
  this->CaptionActor->BorderOn();
  this->CaptionActor->LeaderOn();
  this->CaptionActor->ThreeDimensionalLeaderOn();
  
  this->CaptionGlyph = vtkConeSource::New();
  this->CaptionGlyph->SetResolution(6);
  this->CaptionGlyph->SetCenter(-0.5,0,0);
  this->CaptionActor->SetLeaderGlyph(this->CaptionGlyph->GetOutput());

  this->ShowBorder = vtkBorderRepresentation::BORDER_OFF;
}

//-------------------------------------------------------------------------
vtkCaptionRepresentation::~vtkCaptionRepresentation()
{
  this->SetCaptionActor(0);
  this->CaptionGlyph->Delete();
  this->SetAnchorRepresentation(0);
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::SetAnchorRepresentation(vtkPointHandleRepresentation3D *rep)
{
  if ( rep != this->AnchorRepresentation )
    {
    if ( this->AnchorRepresentation )
      {
      this->AnchorRepresentation->Delete();
      }
    this->AnchorRepresentation = rep;
    if ( this->AnchorRepresentation )
      {
      this->AnchorRepresentation->Register(this);
      }
    this->Modified();
    }
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::SetAnchorPosition(double pos[3])
{
  this->CaptionActor->GetAttachmentPointCoordinate()->SetValue(pos);
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::GetAnchorPosition(double pos[3])
{
  this->CaptionActor->GetAttachmentPointCoordinate()->GetValue(pos);
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::AddProps()
{
  if ( !this->Renderer )
    {
    return;
    }
  this->Renderer->AddViewProp(this->CaptionActor);
  this->Superclass::AddProps();
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::RemoveProps()
{
  if ( !this->Renderer )
    {
    return;
    }
  this->Renderer->RemoveViewProp(this->CaptionActor);
  this->Superclass::RemoveProps();
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::SizeHandles()
{
  // The hardest part is to determine the bounding box for the handle
  double radius, bounds[6], apt[3];
  double windowLowerLeft[4], windowUpperRight[4];
  double *viewport = this->Renderer->GetViewport();
  int *winSize = this->Renderer->GetRenderWindow()->GetSize();

  double x = winSize[0] * viewport[0];
  double y = winSize[1] * viewport[1];
  vtkInteractorObserver::ComputeDisplayToWorld(this->Renderer, x,y,0.0, windowLowerLeft);

  x = winSize[0] * viewport[2];
  y = winSize[1] * viewport[3];
  vtkInteractorObserver::ComputeDisplayToWorld(this->Renderer, x,y,0.0, windowUpperRight);

  int i;
  for (radius=0.0, i=0; i<3; i++) 
    {
    radius += (windowUpperRight[i] - windowLowerLeft[i]) *
      (windowUpperRight[i] - windowLowerLeft[i]);
    }

  this->GetAnchorPosition(apt);
  radius = this->HandleSize*sqrt(radius);
  bounds[0] = apt[0] - radius;
  bounds[1] = apt[0] + radius;
  bounds[2] = apt[1] - radius;
  bounds[3] = apt[1] + radius;
  bounds[4] = apt[2] - radius;
  bounds[5] = apt[2] + radius;
  this->AnchorRepresentation->PlaceWidget(bounds);
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::BuildRepresentation()
{
  // Ask the superclass the size and set the caption
  int *pos1 = this->PositionCoordinate->
    GetComputedDisplayValue(this->Renderer);
  int *pos2 = this->Position2Coordinate->
    GetComputedDisplayValue(this->Renderer);

  if ( this->CaptionActor )
    {
    this->CaptionActor->GetPositionCoordinate()->SetValue(pos1[0],pos1[1]);
    this->CaptionActor->GetPosition2Coordinate()->SetValue(pos2[0],pos2[1]);
    }

  // Note that the transform is updated by the superclass
  this->Superclass::BuildRepresentation();
}

//-------------------------------------------------------------------------
void vtkCaptionRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
  
  os << indent << "Caption Actor: " << this->CaptionActor << "\n";
  
}
