/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkSliderRepresentation.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include "vtkSliderRepresentation.h"
#include "vtkCommand.h"
#include "vtkObjectFactory.h"
#include "vtkActor.h"
#include "vtkPolyDataMapper.h"
#include "vtkCellArray.h"
#include "vtkCylinderSource.h"
#include "vtkCylinder.h"
#include "vtkSphereSource.h"
#include "vtkCellPicker.h"
#include "vtkProperty.h"
#include "vtkRenderer.h"
#include "vtkMath.h"
#include "vtkLine.h"
#include "vtkAssemblyPath.h"
#include "vtkVectorText.h"
#include "vtkBox.h"
#include "vtkAssembly.h"
#include "vtkMatrix4x4.h"
#include "vtkTransform.h"
#include "vtkTransformPolyDataFilter.h"
#include "vtkCamera.h"
#include "vtkEvent.h"
#include "vtkInteractorObserver.h"

vtkCxxRevisionMacro(vtkSliderRepresentation, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkSliderRepresentation);

//----------------------------------------------------------------------
vtkSliderRepresentation::vtkSliderRepresentation()
{
  this->MinimumValue = 0.0;
  this->Value   = 0.0;
  this->MaximumValue = 1.0;
  this->CurrentT = 0.0;
  
  this->SliderShape = vtkSliderRepresentation::SphereShape;
  this->SliderLength = 0.05;
  this->SliderWidth = 0.05;
  this->EndCapLength = 0.025;
  this->TubeWidth = 0.025;
  this->Rotation = 0.0;

  // The cylinder used to model various parts of the widget
  // (caps, tube, and possible the slider)
  this->CylinderSource = vtkCylinderSource::New();
  this->CylinderSource->SetResolution(16);
  this->CylinderSource->SetCenter(0.0,0.0,0.0);
  this->CylinderSource->SetRadius(0.5);
  this->CylinderSource->SetHeight(1.0);

  vtkTransform *xform = vtkTransform::New();
  xform->RotateZ(90.0);
  this->Cylinder = vtkTransformPolyDataFilter::New(); //aling the axis along the x-axis
  this->Cylinder->SetInput(this->CylinderSource->GetOutput());
  this->Cylinder->SetTransform(xform);
  xform->Delete();

  // The tube (the slider moves along the tube)
  this->TubeMapper = vtkPolyDataMapper::New();
  this->TubeMapper->SetInput(this->CylinderSource->GetOutput());
  
  this->TubeProperty = vtkProperty::New();
  this->TubeProperty->SetColor(1,1,1);

  this->TubeActor = vtkActor::New();
  this->TubeActor->SetMapper(this->TubeMapper);
  this->TubeActor->SetProperty(this->TubeProperty);
  this->TubeActor->RotateZ(90.0);
  
  // The slider (either sphere or cylinder)
  this->SliderSource = vtkSphereSource::New();
  this->SliderSource->SetPhiResolution(8);
  this->SliderSource->SetThetaResolution(16);
  this->SliderSource->SetRadius(0.5);
  
  this->SliderMapper = vtkPolyDataMapper::New();
  this->SliderMapper->SetInput(this->SliderSource->GetOutput());
  
  this->SliderProperty = vtkProperty::New();
  this->SliderProperty->SetColor(0.2000, 0.6300, 0.7900); //peacock
  this->SliderProperty->SetSpecularColor(1, 1, 1);
  this->SliderProperty->SetSpecular(0.3);
  this->SliderProperty->SetSpecularPower(20);
  this->SliderProperty->SetAmbient(0.2);
  this->SliderProperty->SetDiffuse(0.8);

  this->SliderActor = vtkActor::New();
  this->SliderActor->SetMapper(this->SliderMapper);
  this->SliderActor->SetProperty(this->SliderProperty);

  // Position the sphere (slider) and compute some intermediate information
  this->SP1[0] = -0.5 + this->EndCapLength + this->SliderLength/2.0;
  this->SP1[1] = 0.0;
  this->SP1[2] = 0.0;
  this->SP2[0] = -0.5 + (1.0-this->EndCapLength) - this->SliderLength/2.0;
  this->SP2[1] = 0.0;
  this->SP2[2] = 0.0;
  
  this->SelectedProperty = vtkProperty::New();
  this->SelectedProperty->SetColor(1.0000, 0.4118, 0.7059); //hot pink
  this->SelectedProperty->SetSpecularColor(1, 1, 1);
  this->SelectedProperty->SetSpecular(0.3);
  this->SelectedProperty->SetSpecularPower(20);
  this->SelectedProperty->SetAmbient(0.2);
  this->SelectedProperty->SetDiffuse(0.8);

  // The left cap
  this->LeftCapMapper = vtkPolyDataMapper::New();
  this->LeftCapMapper->SetInput(this->Cylinder->GetOutput());
  
  this->CapProperty = vtkProperty::New();
  this->CapProperty->SetColor(1,1,1);
  this->CapProperty->SetSpecularColor(1, 1, 1);
  this->CapProperty->SetSpecular(0.3);
  this->CapProperty->SetSpecularPower(20);
  this->CapProperty->SetAmbient(0.2);
  this->CapProperty->SetDiffuse(0.8);

  this->LeftCapActor = vtkActor::New();
  this->LeftCapActor->SetMapper(this->LeftCapMapper);
  this->LeftCapActor->SetProperty(this->CapProperty);
  
  // The right cap
  this->RightCapMapper = vtkPolyDataMapper::New();
  this->RightCapMapper->SetInput(this->Cylinder->GetOutput());
  
  this->RightCapActor = vtkActor::New();
  this->RightCapActor->SetMapper(this->RightCapMapper);
  this->RightCapActor->SetProperty(this->CapProperty);

  this->Point1Coordinate = vtkCoordinate::New();
  this->Point1Coordinate->SetCoordinateSystemToWorld();
  this->Point1Coordinate->SetValue(-1.0,0.0,0.0);

  this->Point2Coordinate = vtkCoordinate::New();
  this->Point2Coordinate->SetCoordinateSystemToWorld();
  this->Point2Coordinate->SetValue(1.0,0.0,0.0);

  // Labels and text
  this->ShowSliderLabel = 1;
  this->LabelText = vtkVectorText::New();
  this->LabelText->SetText("");
  this->LabelMapper = vtkPolyDataMapper::New();
  this->LabelMapper->SetInput(this->LabelText->GetOutput());
  this->LabelActor = vtkActor::New();
  this->LabelActor->SetMapper(this->LabelMapper);
  this->LabelActor->PickableOff();
  
  this->LabelHeight = 0.05;
  this->TitleHeight = 0.15;
  this->TitleText = vtkVectorText::New();
  this->TitleText->SetText("");
  this->TitleMapper = vtkPolyDataMapper::New();
  this->TitleMapper->SetInput(this->TitleText->GetOutput());
  this->TitleActor = vtkActor::New();
  this->TitleActor->SetMapper(this->TitleMapper);
  this->TitleActor->PickableOff();

  this->P1[0] = -1.0;
  this->P1[1] =  0.0;
  this->P1[2] =  0.0;
  this->P2[0] =  1.0;
  this->P2[1] =  0.0;
  this->P2[2] =  0.0;
  
  // Finally, the assembly that holds everything together
  this->WidgetAssembly = vtkAssembly::New();
  this->WidgetAssembly->AddPart(this->TubeActor);
  this->WidgetAssembly->AddPart(this->SliderActor);
  this->WidgetAssembly->AddPart(this->LeftCapActor);
  this->WidgetAssembly->AddPart(this->RightCapActor);
  this->WidgetAssembly->AddPart(this->LabelActor);
  this->WidgetAssembly->AddPart(this->TitleActor);
  
  //Manage the picking stuff
  this->Picker = vtkCellPicker::New();
  this->Picker->SetTolerance(0.001);
  this->Picker->AddPickList(this->WidgetAssembly);
  this->Picker->PickFromListOn();

  this->Matrix = vtkMatrix4x4::New();
  this->Transform = vtkTransform::New();
}

//----------------------------------------------------------------------
vtkSliderRepresentation::~vtkSliderRepresentation()
{
  this->WidgetAssembly->Delete();

  this->CylinderSource->Delete();
  this->Cylinder->Delete();

  this->TubeMapper->Delete();
  this->TubeActor->Delete();
  
  this->SliderSource->Delete();
  this->SliderMapper->Delete();
  this->SliderActor->Delete();

  this->SliderProperty->Delete();
  this->SelectedProperty->Delete();
    
  this->LeftCapMapper->Delete();
  this->LeftCapActor->Delete();
  this->CapProperty->Delete();
  
  this->RightCapMapper->Delete();
  this->RightCapActor->Delete();
  
  this->Point1Coordinate->Delete();
  this->Point2Coordinate->Delete();
  
  this->Picker->Delete();
  
  this->LabelText->Delete();
  this->LabelMapper->Delete();
  this->LabelActor->Delete();
  this->TitleText->Delete();
  this->TitleMapper->Delete();
  this->TitleActor->Delete();
  
  this->Matrix->Delete();
  this->Transform->Delete();
}

//-------------------------------------------------------------------------
void vtkSliderRepresentation::AddProps(vtkRenderer *ren)
{
  ren->AddViewProp(this->WidgetAssembly);
}

//-------------------------------------------------------------------------
void vtkSliderRepresentation::RemoveProps(vtkRenderer *ren)
{
  ren->RemoveViewProp(this->WidgetAssembly);
}

//----------------------------------------------------------------------
vtkCoordinate *vtkSliderRepresentation::GetPoint1Coordinate()
{
  return this->Point1Coordinate;
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::StartWidgetInteraction(vtkRenderer*, double*)
{
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::WidgetInteraction(vtkRenderer*, double*)
{
}

//----------------------------------------------------------------------
int vtkSliderRepresentation::ComputeInteractionState(vtkRenderer*, int, int)
{
  return 0;
}

//----------------------------------------------------------------------
double vtkSliderRepresentation::GetPickedT()
{
  return this->PickedT;
}


//----------------------------------------------------------------------
vtkCoordinate *vtkSliderRepresentation::GetPoint2Coordinate()
{
  return this->Point2Coordinate;
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::SetTitleText(const char* label)
{
  this->TitleText->SetText(label);
}

//----------------------------------------------------------------------
const char* vtkSliderRepresentation::GetTitleText()
{
  return this->TitleText->GetText();
}

#if 0
//----------------------------------------------------------------------
void vtkSliderRepresentation::PlaceWidget()
{
  // See if we can compute bounds
 if ( !this->Prop3D && !this->Input )
    {
    double *p1, *p2;
    if ( this->CurrentRenderer )
      {
      p1 = this->Point1Coordinate->GetComputedWorldValue(this->CurrentRenderer);
      p2 = this->Point2Coordinate->GetComputedWorldValue(this->CurrentRenderer);
      }
    else
      {
      p1 = this->Point1Coordinate->GetValue();
      p2 = this->Point2Coordinate->GetValue();
      }
    double bounds[6];
    for (int i=0; i<3; i++)
      {
      if ( p1[i] < p2[i] )
        {
        bounds[2*i] = p1[i];
        bounds[2*i+1] = p2[i];
        }
      else
        {
        bounds[2*i] = p2[i];
        bounds[2*i+1] = p1[i];
        }
      }
    this->PlaceWidget(bounds);
    }
 
 else //compute the appropriate bounds
   {
   this->Superclass::PlaceWidget();
   }
}


//----------------------------------------------------------------------
void vtkSliderRepresentation::PlaceWidget(double bds[6])
{
  int i;
  double bounds[6], center[3];

  double placeFactor = this->PlaceFactor;
  this->PlaceFactor = 1.0;
  this->AdjustBounds(bds, bounds, center);
  this->PlaceFactor = placeFactor;
  
  for (i=0; i<6; i++)
    {
    this->InitialBounds[i] = bounds[i];
    }
  this->InitialLength = sqrt((bounds[1]-bounds[0])*(bounds[1]-bounds[0]) +
                             (bounds[3]-bounds[2])*(bounds[3]-bounds[2]) +
                             (bounds[5]-bounds[4])*(bounds[5]-bounds[4]));

  // When PlaceWidget is invoked, the widget orientation is preserved, but it
  // is allowed to translate and scale. This means it is centered in the
  // bounding box, and the scalar widget scales itself to intersect the sides
  // of the bounding box. Thus we have to determine where Point1 and Point2
  // intersect the bounding box.
  double *p1, *p2, r[3], o[3], t;
  if ( this->CurrentRenderer )
    {
    p1 = this->Point1Coordinate->GetComputedWorldValue(this->CurrentRenderer);
    p2 = this->Point2Coordinate->GetComputedWorldValue(this->CurrentRenderer);
    }
  else
    {
    p1 = this->Point1Coordinate->GetValue();
    p2 = this->Point2Coordinate->GetValue();
    }

  // Okay, this looks really weird, we are shooting rays from OUTSIDE
  // the bounding box back towards it. This is because the IntersectBox()
  // method computes intersections only if the ray originates outside the
  // bounding box.
  r[0] = this->InitialLength * (p1[0] - p2[0]);
  r[1] = this->InitialLength * (p1[1] - p2[1]);
  r[2] = this->InitialLength * (p1[2] - p2[2]);
  o[0] = center[0] - r[0];
  o[1] = center[1] - r[1];
  o[2] = center[2] - r[2];
  vtkBox::IntersectBox(bounds,o,r,this->P1,t);

  r[0] = this->InitialLength * (p2[0] - p1[0]);
  r[1] = this->InitialLength * (p2[1] - p1[1]);
  r[2] = this->InitialLength * (p2[2] - p1[2]);
  o[0] = center[0] - r[0];
  o[1] = center[1] - r[1];
  o[2] = center[2] - r[2];
  vtkBox::IntersectBox(bounds,o,r,this->P2,t);

  // Position the handles at the end of the lines
  this->BuildRepresentation();
  this->SizeHandles();
}
#endif

//----------------------------------------------------------------------
void vtkSliderRepresentation::SetMinimumValue(double minValue)
{
  if ( minValue == this->MinimumValue )
    {
    return;
    }
  
  if ( minValue >= this->MaximumValue )
    {
    this->MaximumValue = minValue + 1;
    }
  
  this->MinimumValue = minValue;

  if ( this->Value < this->MinimumValue )
    {
    this->Value = this->MinimumValue;
    this->CurrentT = 0.0;
    }
  else if ( this->Value > this->MaximumValue )
    {
    this->Value = this->MaximumValue;
    this->CurrentT = 1.0;
    }

  this->InvokeEvent(vtkCommand::WidgetValueChangedEvent,NULL);
  this->BuildRepresentation();
  this->Modified();
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::SetMaximumValue(double maxValue)
{
  if ( maxValue == this->MaximumValue )
    {
    return;
    }
  
  if ( maxValue <= this->MinimumValue )
    {
    this->MinimumValue = maxValue - 1;
    }
  
  this->MaximumValue = maxValue;

  if ( this->Value < this->MinimumValue )
    {
    this->Value = this->MinimumValue;
    this->CurrentT = 0.0;
    }
  else if ( this->Value > this->MaximumValue )
    {
    this->Value = this->MaximumValue;
    this->CurrentT = 1.0;
    }

  this->InvokeEvent(vtkCommand::WidgetValueChangedEvent,NULL);
  this->BuildRepresentation();
  this->Modified();
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::SetValue(double value)
{
  if ( value == this->Value )
    {
    return;
    }
  
  if ( value < this->MinimumValue )
    {
    value = this->MinimumValue;
    }
  
  if ( value > this->MaximumValue )
    {
    value = this->MaximumValue;
    }
  
  this->Value = value;
  this->CurrentT = (value - this->MinimumValue) / (this->MaximumValue - this->MinimumValue);

  this->InvokeEvent(vtkCommand::WidgetValueChangedEvent,NULL);
  this->BuildRepresentation();
  this->Modified();
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::EvaluateSliderPosition(double t, double pos[3])
{
  double xpos[4], newPos[4];
  xpos[0] = this->SP1[0] + t*(this->SP2[0] - this->SP1[0]);
  xpos[1] = this->SP1[1] + t*(this->SP2[1] - this->SP1[1]);
  xpos[2] = this->SP1[2] + t*(this->SP2[2] - this->SP1[2]);
  xpos[3] = 1.0;

  this->Transform->TransformPoint(xpos,newPos);
  pos[0] = newPos[0];
  pos[1] = newPos[1];
  pos[2] = newPos[2];
}

//----------------------------------------------------------------------
double vtkSliderRepresentation::ComputePickPosition(vtkRenderer *ren, double eventPos[2])
{
  // Transform current pick ray into canonical (untransformed)
  // widget coordinates. This requires a camera.
  vtkCamera *camera = ren->GetActiveCamera();
  if ( !camera )
    {
    return 0.0;
    }

  // The pick ray is defined by the camera position and the (X,Y)
  // pick position in the renderer. The depth of the (X,Y) pick is
  // is the back clipping plane.
  double cameraWorldPosition[4], cameraPosition[4];
  camera->GetPosition(cameraWorldPosition); 
  cameraWorldPosition[3] = 1.0;
  this->Transform->TransformPoint(cameraWorldPosition,cameraPosition);

  double rayEndPoint[4], rayPosition[4];
  vtkInteractorObserver::ComputeDisplayToWorld(ren, eventPos[0], eventPos[1], 1.0, rayEndPoint);
  this->Transform->TransformPoint(rayEndPoint,rayPosition);

  // Now intersect the two lines and compute the pick position
  // along the slider.
  double u, v;
  vtkLine::Intersection(this->SP1,this->SP2, cameraPosition,rayPosition, u,v);

  return u;
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::Highlight(int highlight)
{
  if ( highlight )
    {
    this->SliderActor->SetProperty(this->SelectedProperty);
    }
  else
    {
    this->SliderActor->SetProperty(this->SliderProperty);
    }
}


//----------------------------------------------------------------------
void vtkSliderRepresentation::BuildRepresentation()
{
  double sx, sy;
  double t = (this->Value-this->MinimumValue) / (this->MaximumValue-this->MinimumValue);

  // Setup the geometry of the widget (canonical -0.5 to 0.5 along the x-axis)
  this->SP1[0] = -0.5 + this->EndCapLength + this->SliderLength/2.0;
  this->SP1[1] = 0.0;
  this->SP1[2] = 0.0;
  this->SP2[0] = -0.5 + (1.0-this->EndCapLength) - this->SliderLength/2.0;
  this->SP2[1] = 0.0;
  this->SP2[2] = 0.0;

  // The end points P1,P2 are assumed set by PlaceWidget() or other
  this->Length = sqrt (vtkMath::Distance2BetweenPoints(this->P1,this->P2));
  if ( this->Length <= 0.0 )
    {
    this->P2[0] = this->P1[0] + 1.0; this->P2[1] = 0.0; this->P2[2] = 0.0;
    }
  
  // Update the canonical shape of the widget
  if ( this->SliderShape == vtkSliderRepresentation::SphereShape )
    {
    this->SliderMapper->SetInput(this->SliderSource->GetOutput());
    }
  else
    {
    this->SliderMapper->SetInput(this->Cylinder->GetOutput());
    }

  this->TubeActor->SetScale(this->TubeWidth, 1.0-(2.0*this->EndCapLength), this->TubeWidth);
  this->LeftCapActor->SetPosition(-0.5+(this->EndCapLength/2.0),0,0); 
  this->LeftCapActor->SetScale(2.0*this->TubeWidth, this->EndCapLength, 
                               2.0*this->TubeWidth);
  this->RightCapActor->SetPosition(0.5-(this->EndCapLength/2.0),0,0); 
  this->RightCapActor->SetScale(2.0*this->TubeWidth, this->EndCapLength, 
                                2.0*this->TubeWidth);
  if ( this->EndCapLength <= 0.0 )
    {
    this->RightCapActor->VisibilityOff();
    this->LeftCapActor->VisibilityOff();
    }
  else
    {
    this->RightCapActor->VisibilityOn();
    this->LeftCapActor->VisibilityOn();
    }

  // Position the slider (sphere)
  //
  double p[3];
  p[0] = this->SP1[0] + t*(this->SP2[0]-this->SP1[0]);
  p[1] = this->SP1[1] + t*(this->SP2[1]-this->SP1[1]);
  p[2] = this->SP1[2] + t*(this->SP2[2]-this->SP1[2]);
  this->SliderActor->SetPosition(p);
  this->SliderActor->SetScale(this->SliderLength,this->SliderWidth,this->SliderWidth);

  // Here we position the title and the slider label. Of course this is a 
  // function of the text strings that ave been supplied.
  // Place the title
  if ( this->TitleText->GetText() == NULL ||
       *(this->TitleText->GetText()) == '\0' )
    {
    this->TitleActor->VisibilityOff();
    }
  else
    {
    double bounds[6];
    this->TitleActor->VisibilityOn();
    this->TitleText->Update();
    this->TitleText->GetOutput()->GetBounds(bounds);

    sy = this->TitleHeight / (bounds[3]-bounds[2]);
    sx = sy;

    // Compute translation: first, where the current center is
    // (We want to perform scaling and rotation around origin.)
    double c1[3], c2[3];
    c1[0] = (bounds[1]+bounds[0])/2.0;
    c1[1] = (bounds[3]+bounds[2])/2.0;
    c1[2] = (bounds[5]+bounds[4])/2.0;
    
    // Where we want the center to be
    c2[0] = (this->SP1[0]+this->SP2[0])/2.0;
    c2[1] = (this->SP1[1]+this->SP2[1])/2.0 - 2.0*sy;
    c2[2] = (this->SP1[2]+this->SP2[2])/2.0;
    
    // Transform the text
    this->TitleActor->SetOrigin(c1[0], c1[1], c1[2]);
    this->TitleActor->SetScale(sx,sy,1.0);
    this->TitleActor->SetPosition(c2[0]-c1[0], c2[1]-c1[1], c2[2]-c1[2]);
    }

  // Place the slider label
  if ( ! this->ShowSliderLabel )
    {
    this->LabelActor->VisibilityOff();
    }
  else
    {
    char label[256];
    sprintf(label,"%0.3g",this->Value);
    double bounds[6];
    this->LabelActor->VisibilityOn();
    this->LabelText->SetText(label);
    this->LabelText->Update();
    this->LabelText->GetOutput()->GetBounds(bounds);

    //scaling
    sy = this->LabelHeight / (bounds[3] - bounds[2]);
    sx = sy;

    // Compute translation: first, where the current center is
    // (We want to perform scaling and rotation around origin.)
    double c1[3], c2[3];
    c1[0] = (bounds[1]+bounds[0])/2.0;
    c1[1] = (bounds[3]+bounds[2])/2.0;
    c1[2] = (bounds[5]+bounds[4])/2.0;
    
    // Where we want the center to be
    c2[0] = this->SP1[0] + t*(this->SP2[0]-this->SP1[0]);
    c2[1] = this->SP1[1] + t*(this->SP2[1]-this->SP1[1]) + 2.0*sy;
    c2[2] = this->SP1[2] + t*(this->SP2[2]-this->SP1[2]);

    // Actor the text
    this->LabelActor->SetOrigin(c1[0], c1[1], c1[2]);
    this->LabelActor->SetScale(sx,sy,1.0);
    this->LabelActor->SetPosition(c2[0]-c1[0], c2[1]-c1[1], c2[2]-c1[2]);
    }

  // Compute the rotation of the widget. Note that the widget as constructed 
  // is oriented in the x-direction. Here we rotate the whole assembly.
  double x[3], v[3], axis[3];
  x[0] = 1.0; 
  x[1] = 0.0; 
  x[2] = 0.0; 
  v[0] = this->P2[0] - this->P1[0]; 
  v[1] = this->P2[1] - this->P1[1]; 
  v[2] = this->P2[2] - this->P1[2];
  vtkMath::Normalize(v);
  vtkMath::Cross(v,x,axis);
  double theta, axisLen = vtkMath::Norm(axis);
  if ( axisLen != 0.0 )
    {
    theta = asin(axisLen) * vtkMath::DoubleRadiansToDegrees();
    }
  else
    {
    theta = 0.0;
    axis[0] = 0.0; axis[1] = 1.0; axis[2] = 0.0;
    }
  this->WidgetAssembly->SetOrientation(0.0,0.0,0.0);
  this->WidgetAssembly->RotateX(this->Rotation);
  this->WidgetAssembly->RotateWXYZ(theta,axis[0],axis[1],axis[2]);
  this->WidgetAssembly->SetScale(this->Length, this->Length, this->Length);
  p[0] = (this->P1[0] + this->P2[0])/2.0;
  p[1] = (this->P1[1] + this->P2[1])/2.0;
  p[2] = (this->P1[2] + this->P2[2])/2.0;
  this->WidgetAssembly->SetPosition(p);

  // A final task: get the transformation matrix for the "tube"
  this->Transform->Pop();
  this->WidgetAssembly->GetMatrix(this->Matrix);
  this->Transform->SetMatrix(this->Matrix);
  this->Transform->Push();
  this->Transform->Inverse();
}

//----------------------------------------------------------------------
void vtkSliderRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  //Superclass typedef defined in vtkTypeMacro() found in vtkSetGet.h
  this->Superclass::PrintSelf(os,indent);

  os << indent << "Point1 Coordinate: " << this->Point1Coordinate << "\n";
  this->Point1Coordinate->PrintSelf(os, indent.GetNextIndent());

  os << indent << "Point2 Coordinate: " << this->Point2Coordinate << "\n";
  this->Point2Coordinate->PrintSelf(os, indent.GetNextIndent());

  if ( this->SliderProperty )
    {
    os << indent << "Slider Property:\n";
    this->SliderProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "Slider Property: (none)\n";
    }

  if ( this->SelectedProperty )
    {
    os << indent << "SelectedProperty:\n";
    this->SelectedProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "SelectedProperty: (none)\n";
    }

  if ( this->TubeProperty )
    {
    os << indent << "TubeProperty:\n";
    this->TubeProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "TubeProperty: (none)\n";
    }

  if ( this->CapProperty )
    {
    os << indent << "CapProperty:\n";
    this->CapProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "CapProperty: (none)\n";
    }

  os << indent << "Minimum Value: " << this->MinimumValue << "\n";
  os << indent << "Maximum Value: " << this->MaximumValue << "\n";
  os << indent << "Value: " << this->Value << "\n";
  
  if ( this->SelectedProperty )
    {
    os << indent << "SelectedProperty:\n";
    this->SelectedProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "SelectedProperty: (none)\n";
    }

  if ( this->SliderShape == vtkSliderRepresentation::SphereShape )
    {
    os << indent << "Slider Shape: Sphere\n";
    }
  else
    {
    os << indent << "Slider Shape: Cylinder\n";
    }
  
  os << indent << "Slider Length: " << this->SliderLength << "\n";
  os << indent << "Slider Width: " << this->SliderWidth << "\n";
  os << indent << "End Cap Length: " << this->EndCapLength << "\n";
  os << indent << "Tube Width: " << this->TubeWidth << "\n";
  os << indent << "Rotation: " << this->Rotation << "\n";

  os << indent << "Show Slider Label: " 
     << (this->ShowSliderLabel ? "On\n" : "Off\n");
  os << indent << "Label Height: " << this->LabelHeight << "\n";
  os << indent << "Title Height: " << this->TitleHeight << "\n";
  os << indent << "Title Text: " << (this->TitleText->GetText() ? 
                                     this->TitleText->GetText() : 
                                     "(none)") << "\n";
}
